//trieda ExtendedFATFileSystem -- rozsiruje funkcionalitu FATFileSystem
//LINUX gcc verzia
//pouziva triedy so STL kniznice

#ifndef ExtendedFATFileSystemH
#define ExtendedFATFileSystemH

#include "fatlib.h"
#include "xcompat.h"
#include <string>
#include <deque>

using namespace std;

//nasledujuce dve struktury sa pouzivaju pri ziskavani obsahu existujuceho EFS
//tato struktura obsahuje informacie o jednej polozke adresara
struct DirEnt
{
string Name; //obsahuje meno polozky, dlhe (ak ex., inak kratke)
DirectoryEntry de; //vsetky ostatne udaje
DirEnt(void);
DirEnt(char *Name, DirectoryEntry *de);
DirEnt(const DirEnt &de); //kopirovaci konstruktor
~DirEnt(void);
};

//tato struktura obsahuje informacie o jednom adresari EFS
struct DirDescr
{
string DirName;
deque<DirEnt> DirEntries;
};


//tieto dve struktury sa pouzivaju na popis pouzitia sablon pri generovani nahodneho EFS
//pomocna struktura
struct DirDesc
	{
	string DirName;
	deque<string> Files_q;
	};

//tato struktura popisuje pouzitie sablon a rozmiestnenie suborov z nich generovanych v EFS:
struct FileTemplDesc
	{
	string Template;
	deque<DirDesc> Dirs_q;
	};



class ExtendedFATFileSystem : public FATFileSystem
	{
	public:
	ExtendedFATFileSystem(void);
	~ExtendedFATFileSystem(void);

	// funkcia ulozi obsah lub. adresara do fronty a vrati true(ak adresar v EFS ex.)
	//inak vrati 0
	//ako parameter sa zada meno adresara(spatne lomitka), smernik na existujucu frontu, jej pripadny obsah
	//bude zmazany, DeletedEntries - ak je to true, funkcia zahrnie aj zmazane polozky (len kratke n.)
	bool GetEFSDir(char *DirName, deque<DirEnt> *DirContent, bool DeletedEntries=false);

	//funkcia do fronty ulozi obsah celeho EFS(vsetky subory/adresare, aj zmazane polozky)
	//root je prvy v poradi oznaceny prazdnym retazcom
	void GetAllEFSDirs(deque<DirDescr> *Result);

	//FUNKCIE NA GENEROVANIE NAHODNYCH MIEN SUBOROV, pripravu nahodnych EFS apod - nevyzaduju inic. EFS

	/*
	funkcia naplni existujucu frontu result(pripadny obsah bude zmazany) Num sablonami mien suborov:
	base - zaklad pre sablonu: znaky # budu nahradene:
	-metaznakom (*, ?) s pravdep. ProbM %, pricom
		* ma pravdep. ProbA% (? 100-ProbA %)
	-nahodnym pismenom alebo cislicou s pravdep. (100 - ProbM %)
	-NoMChars - ak je true, tak sa toleruju sablony bez metaznakov
	-sablony len s jednym alebo 2 ? sa netoleruju
	*/
	void GenerateFileNameTemplates(deque<string> *Result, string Base, int Num, int ProbM, int ProbA, bool NoMChars=false);
	
	/* podobna funkcia ako predosla, generuje sablony pre nazvy Num sablon pre kratke nazvy v tvare ########.###
	do sablony umiestni 1 * a pripadne niekolko ? tak, aby dlzka nepresiahla 11 znakov za predpokl, ze * sa nahradi
	0-3 znakmi, zabezpecuje jedinecnost generovanych sablon*/
	void GenerateShortFileNameTemplates(deque<string> *Result, int Num);

	/*
	funkcia naplni existujucu frontu result(pripadny obsah bude zmazany) menami suborov gener. zo
	sablon vo fronte Templates: z kazdej sablony bude gener. aspon Nmin mien, ale nie viac ako Nmax mien:
	znak * bude v sablone nahradeny 0-10 nah. znakmi/cislicami
	znak ? bude v sablone nahradeny nahodnym znakom/cislicou
	mena nemusia byt jedinecne
	MaxAstCharCnt urcuje maximalny pocet znakov, ktorymi bude nahradena *
	*/
	void GenerateFNamesFromTmplts(deque<string> *Result, deque<string> *Templates, int Nmin, int Nmax, int MaxAstCharCnt=10);

	/* funkcia vygenruje subory s nahodnym obsahom - velkost medzi minsize-maxsize
	path - bude pripojena pred meno suboru z fronty Names, prazdne retazce sa vynechaju ..
	vrati pocet gener. suborov -- pochopitelne do realneho FS
	*/
	int GenerateRandomFiles(string path, deque<string> *Names, int minsize, int maxsize);

	/*
	tato funkcia vytvori strukturu nahodneho EFS
	vsetky fronty musia existovat
	Result - vysledok, pouziva sa pri niekt. kontrolach ---VYSTUP
	Templates - fronta sablon -- VSTUP
	FileNames - mena suborov gener. zo sablon -- VSTUP
	Dirs -- zoznam adresarov v EFS s celou cestou --VYSTUP
	Files - -- zoznam vs. suborov v efs aj s cestou VYSTUP
	Attr -- zoznam zodp. nah. atributov najprv pre adresare a potom pre subory -- VYSTUP
	NumOfDirs -- pocet adresarov na vytvorenie v EFS
	ProbSubDir - prav. vnorenia vytv. adresara v %
	ProbFlInDir - prav. vnorenia suboru do adresara
	ProbAR
	ProbAH         pravdepodnosti nastavenia jednotl. atributov RHSA
	ProbAS
	ProbAA
	UseShNames	priznak pouzitia kratkych nazvov
	*/
	void CreateRandomEFSStructure(deque<FileTemplDesc> *Result, deque<string> *Dirs, deque<string> *Files, deque<int> *Attr, deque<string> *Templates, deque<string> *FileNames, int NumOfDirs, int ProbSubDir, int ProbFlInDir, int ProbAR, int ProbAH, int ProbAS, int ProbAA, bool UseShNames=false);

	/* NASLEDUJUCA FUNKCIA vytvori v inic. EFS adresare a subory, priradi im atributy, ako vstupy pouziva vystupy
	   predoslych funkcii
	   VYZADUJE INICIALIZOVANY EFS!!!
	   mozno neskor bude aj log
	   path - cesta k suborom, ktore budu vkladane
	   Files - fronta mien suborov aj s cestami
	   FileNames - fronta mien suborov bez ciest
	   Dirs - fronta mien adresarov aj s cestami
	   Attr - fronta atributov - najprv adresare, potom subory
	   FragmentFiles - priznak fragmentacie vkladanych suborov
	   UseShNames -- priznak pouzitia kratkych nazvov pri vytvarani suborov a adresarov
	*/
	bool FillEFS(string path, deque<string> *Files, deque<string> *FileNames, deque<string> *Dirs, deque<int> *Attr, bool FragmentFiles=false, bool UseShNames=false);
	};

#endif
